<?php

namespace Drupal\domain_menus\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Render\Markup;
use Drupal\system\Entity\Menu;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class DomainMenusSettingsForm.
 */
class DomainMenusSettingsForm extends ConfigFormBase {

  /**
   * Drupal\Core\Entity\EntityTypeManagerInterface definition.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Drupal\Core\Messenger\MessengerInterface definition.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * Config settings.
   *
   * @var string
   */
  const SETTINGS = 'domain_menus.settings';

  /**
   * {@inheritdoc}
   */
  public function __construct(ConfigFactoryInterface $config_factory, MessengerInterface $messenger, EntityTypeManagerInterface $entity_type_manager) {
    parent::__construct($config_factory);
    $this->messenger = $messenger;
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('messenger'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'domain_menus_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      static::SETTINGS,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config(static::SETTINGS);

    $form['domain_menus_menu_names'] = [
      '#type' => 'textarea',
      '#rows' => 5,
      '#columns' => 40,
      '#title' => $this->t('Domain menus'),
      '#default_value' => $config->get('domain_menus_menu_names'),
      '#description' => $this->t('Enter unique menu names/identifiers (less than 10 characters each, alphanumeric only), one per line.'),
    ];

    $form['domain_menus_create'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Operation: Create menus'),
      '#default_value' => 0,
      '#description' => $this->t('Create domain menus (not already existent) for every domain.'),
    ];

    $form['domain_menus_delete'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Operation: Delete menus'),
      '#default_value' => 0,
      '#description' => $this->t('Delete all domain menus and all links in them!'),
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * Returns an array of settings keys.
   */
  public function settingsKeys() {
    return [
      'domain_menus_menu_names',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config(static::SETTINGS);
    foreach ($this->settingsKeys() as $key) {
      $config->set($key, $form_state->getValue($key));
    }
    $config->save();

    if($form_state->getValue('domain_menus_delete')) {
      $menus = $this->entityTypeManager->getStorage('menu')->loadMultiple();
      foreach($menus as $menu) {
        $menu_domains = $menu->getThirdPartySetting("domain_menus", "domains");
        if($menu_domains !== NULL) {
          $menu->delete();
        }
      }
      $this->messenger->addMessage($this->t('Domain menu(s) deleted.'), 'status');
    }

    if($form_state->getValue('domain_menus_create')) {
      $count = 0;
      $domain_menus_menu_names = $form_state->getValue('domain_menus_menu_names');
      if(!empty($domain_menus_menu_names)) {
        $menu_names = explode("\r\n", $domain_menus_menu_names);
        $domains = $this->entityTypeManager->getStorage('domain')->loadMultiple();
        if(!empty($domains)) {
          foreach($domains as $domain) {
            $domain_id = $domain->id();
            $domain_domainid = $domain->getDomainId();
            $domain_label = Markup::create($domain->label());
            foreach($menu_names as $menu_name) {
              if(!empty($menu_name)) {
                $menu_id = trim($this->t(DOMAIN_MENUS_MENU_ID_PATTERN, ['@domain_domainid' => $domain_domainid, '@menu_name' => $menu_name]));
                $menu_label = trim($this->t(DOMAIN_MENUS_MENU_LABEL_PATTERN, ['@domain_label' => $domain_label, '@menu_name' => $menu_name]));
                if(!($menu = Menu::load($menu_id))) {
                  $menu = Menu::create(array(
                    'id' => $menu_id,
                    'label' => $menu_label,
                  ));
                  $count++;
                }
                $menu->setThirdPartySetting('domain_menus', 'domains', array($domain_id => $domain_id));
                $menu->setThirdPartySetting('domain_menus', 'auto-created', 1);
                $menu->save();
              }
            }
          }
        }
      }
      $this->messenger->addMessage($this->t('@count domain menu(s) created.', ['@count'=>$count]), 'status');
    }

    parent::submitForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $domain_menus_menu_names = $form_state->getValue('domain_menus_menu_names');
    if(!empty($domain_menus_menu_names)) {
      $menu_names = explode("\r\n", $domain_menus_menu_names);
      foreach($menu_names as $menu_name) {
        if(strlen($menu_name) > 9 || !ctype_alnum($menu_name)) {
          $form_state->setErrorByName('domain_menus_menu_names', $this->t('Domain menu names must be alphanumeric and less than 10 characters each.'));
        }
      }
    }
  }
}
